# fcForum.py 
# Main fcForum Class

__doc__ = """fcForum Product - Zope Message Boards"""

__version__ = '2.0.7'

#from AccessControl.Role import RoleManager
from AccessControl import ClassSecurityInfo
from AccessControl.PermissionRole import rolesForPermissionOn
#from Acquisition import aq_base
from Globals import DTMLFile
from Globals import InitializeClass
from Globals import Persistent
from OFS.Folder import Folder
from App.FindHomes import INSTANCE_HOME, SOFTWARE_HOME

from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod, ExternalMethod
from Products.MailHost import MailHost
from Products.PythonScripts.PythonScript import manage_addPythonScript, PythonScript 
from Products.PageTemplates.ZopePageTemplate import manage_addPageTemplate
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from webdav.WriteLockInterface import WriteLockInterface


# The only "external dependancy"
from Products.CookieCrumbler import CookieCrumbler

import os
import string
import random
import Acquisition
import AccessControl

# For HTML Parsing/Processing and Search
import sgmllib
import re
from string import whitespace

# For Reverse DNS functions
import socket

# Handles Forums Locations (under fcForum folder)
fcForumsLocation = 'forums'

import fcForumObject

# Constructor interface for the whole forum system
manage_addFcForumForm = PageTemplateFile('www/fcForumAdd.pt', globals())

def manage_addFcForum(self, id, REQUEST=None):
  """Adds a new fcForum Instance"""
  self._setObject(id, fcForum(id, REQUEST))
  if REQUEST.get('add_default_forums'):
    # Change Permissions to created Forums
    changeForumPermissions(self, id)
  # Default Permissions Settings
  changePermissions(self, id)
  return self.manage_main(self, REQUEST)

def gen_id():
  return str(random.random())[2:]

def changeForumPermissions(self, id):
  """Change Forum Default Permissions"""
  admin_id  = 'administration' # '%s1' % gen_id()
  public_id = '%s2' % gen_id()
  member_id = '%s3' % gen_id()

  forum = getattr(self, id)  
      
  forum.fcForum[fcForumsLocation].manage_addFolder(admin_id, 'Administration Forum')
  forum.fcForum[fcForumsLocation].manage_addFolder(member_id, 'Member Test Forum')
  forum.fcForum[fcForumsLocation].manage_addFolder(public_id, 'Public Test Forum')
    
  administrator_forum = getattr(forum.fcForum[fcForumsLocation], admin_id)
  member_forum = getattr(forum.fcForum[fcForumsLocation], member_id)
  public_forum = getattr(forum.fcForum[fcForumsLocation], public_id)
      
  # Admin Auto Forum
  administrator_forum.manage_addProperty(id='fdesc', type='text', value='Administration Forum')
  administrator_forum.manage_permission('Access contents information', ('Manager', 'Reviewer',), 0)
  administrator_forum.manage_permission('View',('Manager', 'Reviewer',),acquire=0)
      
  # Member Auto Forum      
  member_forum.manage_addProperty(id='fdesc', type='text', value='Member Test forum')
  member_forum.manage_permission('Access contents information',('Manager', 'Reviewer', 'Member',),acquire=0)
  member_forum.manage_permission('View',('Manager', 'Reviewer', 'Member',),acquire=0)      
      
  # Public Auto Forum
  public_forum.manage_addProperty(id='fdesc', type='text', value='Public Test forum')
  public_forum.manage_permission('Access contents information', [],acquire=1)
  public_forum.manage_permission('View',[],acquire=1)

def changePermissions(self, id):
  """Manage Permissions for specific published objects"""

  forum = getattr(self, id)

  # Protect Admin Forum
  # PROTO
  #self.admin.manage_permission('Access contents information',('Manager', 'Reviewer', 'Member',),acquire=0)
  #self.admin.manage_permission('View',('Manager', 'Reviewer', 'Member',),acquire=0)
  # Admin Section Must be readable by Reviewers/Managers only
  forum.admin.manage_permission('Access contents information',('Manager', 'Reviewer',),0)
  forum.admin.manage_permission('View',('Manager', 'Reviewer',),0)

  # User List
  forum.fcForum.list_users.manage_permission('Access contents information',('Manager', 'Reviewer', 'Member',),0)
  forum.fcForum.list_users.manage_permission('View',('Manager', 'Reviewer', 'Member',),0)

  # Add Entry Form
  forum.fcForum.add_entry_form.manage_permission('Access contents information',('Manager', 'Reviewer', 'Member',),0)
  forum.fcForum.add_entry_form.manage_permission('View',('Manager', 'Reviewer', 'Member',),0)

# Now this is the main folderish class that contains the whole forum
class fcForum(Folder,
              Acquisition.Implicit,
              AccessControl.Role.RoleManager,
              Persistent,):
  """fcForum Folder(ish)"""
  
  meta_type = 'fcForum'

  security = ClassSecurityInfo()

  def __init__(self, id, REQUEST):
    """Initialize new fcForum"""

    self.id = id
    self.title = REQUEST.get('title', '')
    if REQUEST is not None:
      self.add_all_methods(REQUEST)

  def add_all_methods(self, REQUEST):
    """Init Method Companion"""

    available_skins = 'autumm,default,corporate,crystal,hacker,myblue,mujer,plonish,xettai'

    # Section I - Addition of page templates to main forum instance:
    # --------------------------------------------------------------
    for k in [INSTANCE_HOME, SOFTWARE_HOME]:
      location = os.path.join(k, 'Products', 'fcForum', 'www', 'forum_main')
      if os.path.exists(location):
        files_dir = location
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file[-2:] == 'ps':
        id = file[:-3]
        manage_addPythonScript(self, id=id)
        self._getOb(id).write(f_data)
      elif file[-2:] == 'pt':
        id = file[:-3]
        manage_addPageTemplate(self, id=id, title='', text=f_data)
      elif file[-4:] == 'dtml':
        id = file[:-5]
        self.addDTMLMethod(id=id, title='', file=f_data)    


    # Section 2 - Add necessary "subfolders" on the main forum instance:
    # And Place the corresponding files/scripts in them
    # --------------------------------------------------------------------
     
    folder_list = ['userinterface', 'manual', 'fcForum', 'images', 'admin', 'Members']
    for folder in folder_list:
      self.manage_addFolder(id=folder, title='')
    # Special Cases.. Add the main forum container folder inside "fcForum"
    
    self.fcForum.manage_addFolder(id=fcForumsLocation, title='Forums Container')
    self.Members.manage_addFolder(id='system', title='System Account (Restricted)')
    # Administrator Property Container
    self.Members.manage_addFolder(id='administrator', title='')
    # AVATAR/Temp Property Container
    self.Members.manage_addFolder(id='AVATARS', title='System Account (Restricted)')
    self.Members.AVATARS.manage_addFolder(id='temp', title='Do Not Delete')
    self.images.manage_addFolder(id='emoticons', title='')
    

    # System Folder (Absolutely Needed)
    # You may want to change this to someplace else..
    system = getattr(self.Members, 'system')
    system.manage_addProperty(id='fversion', type='string', value=__version__)
    system.manage_addProperty(id='forum_base', type='string', value=self.id)
    system.manage_addProperty(id='topicsperpage', type='int', value=15)
    system.manage_addProperty(id='email_password', type='int', value=0)
    system.manage_addProperty(id='fc_skins', type='string', value=available_skins)
    # Added 2.0.4 - Modify System-default language (for presentation)
    system.manage_addProperty(id='lang', type='string', value=REQUEST.get('lang', 'en/us'))
    # Added 2.0.5 - Default Avatars Mode - Use Avatars
    system.manage_addProperty(id='use_avatars', type='string', value='1')
    # Added 2.0.6 - Stocl Image "on" by default
    system.manage_addProperty(id='use_stock_image', type='string', value='1')

    # Add the administrator "allow contact via email" flag set to true
    administrator = getattr(self.Members, 'administrator')
    administrator.manage_addProperty(id='fc_allow_email', type='int', value=1)
    if REQUEST.get('admin_email', ''):
      administrator.manage_addProperty(id='fc_email', type='string', value=REQUEST.get('admin_email'))
    

    # Section 3 - Populate each created folder with its contents
    # -----------------------------------------------------------

    # CSS
    for k in [INSTANCE_HOME, SOFTWARE_HOME]:
      location = os.path.join(k, 'Products', 'fcForum', 'www', 'forum_css')
      if os.path.exists(location):
        files_dir = location
    folder_ob = self._getOb('userinterface')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file[-2:] == 'ps':
        id = file[:-3]
        manage_addPythonScript(folder_ob, id=id)
        folder_ob._getOb(id).write(f_data)
      elif file[-2:] == 'pt':
        id = file[:-3]
        manage_addPageTemplate(folder_ob, id=id, title='', text=f_data)
      elif file[-4:] == 'dtml':
        id = file[:-5]
        folder_ob.addDTMLMethod(id=id, title='', file=f_data)

    # Properties API
    for k in [INSTANCE_HOME, SOFTWARE_HOME]:
      location = os.path.join(k, 'Products', 'fcForum', 'www', 'forum_properties_api')
      if os.path.exists(location):
        files_dir = location
    folder_ob = self._getOb('Members')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file[-2:] == 'ps':
        id = file[:-3]
        manage_addPythonScript(folder_ob, id=id)
        folder_ob._getOb(id).write(f_data)
      elif file[-2:] == 'pt':
        id = file[:-3]
        manage_addPageTemplate(folder_ob, id=id, title='', text=f_data)
      elif file[-4:] == 'dtml':
        id = file[:-5]
        folder_ob.addDTMLMethod(id=id, title='', file=f_data)


    # Admin Section
    for k in [INSTANCE_HOME, SOFTWARE_HOME]:
      location = os.path.join(k, 'Products', 'fcForum', 'www', 'forum_admin')
      if os.path.exists(location):
        files_dir = location
    folder_ob = self._getOb('admin')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file[-2:] == 'ps':
        id = file[:-3]
        manage_addPythonScript(folder_ob, id=id)
        folder_ob._getOb(id).write(f_data)
      elif file[-2:] == 'pt':
        id = file[:-3]
        manage_addPageTemplate(folder_ob, id=id, title='', text=f_data)
      elif file[-4:] == 'dtml':
        id = file[:-5]
        folder_ob.addDTMLMethod(id=id, title='', file=f_data)


    # Manual
    for k in [INSTANCE_HOME, SOFTWARE_HOME]:
      location = os.path.join(k, 'Products', 'fcForum', 'www', 'forum_manual')
      if os.path.exists(location):
        files_dir = location
    folder_ob = self._getOb('manual')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file[-2:] == 'ps':
        id = file[:-3]
        manage_addPythonScript(folder_ob, id=id)
        folder_ob._getOb(id).write(f_data)
      elif file[-2:] == 'pt':
        id = file[:-3]
        manage_addPageTemplate(folder_ob, id=id, title='', text=f_data)
      elif file[-4:] == 'dtml':
        id = file[:-5]
        folder_ob.addDTMLMethod(id=id, title='', file=f_data)


    # Master fcForum Section (Guts)
    for k in [INSTANCE_HOME, SOFTWARE_HOME]:
      location = os.path.join(k, 'Products', 'fcForum', 'www', 'forum_guts')
      if os.path.exists(location):
        files_dir = location
    folder_ob = self._getOb('fcForum')
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file))
      f_data = f_han.read()
      f_han.close()
      if file[-2:] == 'ps':
        id = file[:-3]
        manage_addPythonScript(folder_ob, id=id)
        folder_ob._getOb(id).write(f_data)
      elif file[-2:] == 'pt':
        id = file[:-3]
        manage_addPageTemplate(folder_ob, id=id, title='', text=f_data)
      elif file[-4:] == 'dtml':
        id = file[:-5]
        folder_ob.addDTMLMethod(id=id, title='', file=f_data)

    # Images
    for k in [INSTANCE_HOME, SOFTWARE_HOME]:
      location = os.path.join(k, 'Products', 'fcForum', 'www', 'forum_images')
      if os.path.exists(location):
        files_dir = location    
    folder_ob = self.images
    for file in os.listdir(files_dir):
      if file != 'emoticons':
        f_han  = open(os.path.join(files_dir, file), "rb")
        f_data = f_han.read()
        f_han.close()
        folder_ob.manage_addImage(file, f_data)
    # Images - Emoticons
    for k in [INSTANCE_HOME, SOFTWARE_HOME]:
      location = os.path.join(k, 'Products', 'fcForum', 'www', 'forum_images', 'emoticons')
      if os.path.exists(location):
        files_dir = location    
    folder_ob = self.images.emoticons
    for file in os.listdir(files_dir):
      f_han  = open(os.path.join(files_dir, file), "rb")
      f_data = f_han.read()
      f_han.close()
      fname = file[:-4]
      folder_ob.manage_addImage(file, f_data, ':%s:' % fname)
      
    # Ok, at this point, need to do the following.. (in random order)
    # - Set Up Roles (Member, Reviewer)
    # - Set Up security and proxy roles
    # - Add acl_users (Users) Folder
    #   + Add "administrator/administrator", roles:Reviewer
    #   + Add "member/member", roles:Member
    # - Add CookieCrumbler Instance
    # - Add a MailHost instance, allow to change it from admin interface

    # Create Roles:
    for fc_role in ['Member', 'Reviewer']:
      self._addRole(role=fc_role)

    # Add acl_users instance if REQUESTed
    if REQUEST.get('add_user_instance'):
      self.manage_addUserFolder()
      # Create two users on this newly created instance
      user_instance = getattr(self, 'acl_users')
      user_instance._addUser('administrator', 'administrator', 'administrator', ['Reviewer'], '')
      user_instance._addUser('member', 'member', 'member', ['Member'], '')

      # Add CookieCrumbler Instance
      CookieCrumbler.manage_addCC(self, 'cookie_authentication')

    # MailHost Settings, only mail_host is checked, SMTP defaults to 25 always
    if REQUEST.get('mail_host', ''):
      MailHost.manage_addMailHost(self, 'MailHost', '', REQUEST.get('mail_host'))
    else:
      MailHost.manage_addMailHost(self, 'MailHost')     

    # Management Interface for the fcForum Instance....
    # NOT IMPLEMENTED YET


  # Compile a list of methods that
  # require a proxy role and then create the interfaces
  # to handle them:

  # Membership API: START
  # ===================================================================================
  # 
  security.declarePublic('addFcForumObject')
  # There sould probably abe a permission for this, but oh well..
  def addFcForumObject(self, forum_loc, REQUEST):
    return fcForumObject.manage_addFcForumObject(self, forum_loc, REQUEST)

  # Change selected topic attributes from a dict
  security.declarePublic('changeTopicProperties')
  def changeTopicProperties(self, forum_id, topic_id, attrs, REQUEST):
    fcForumObject.manage_changeTopicProperties(self, forum_id, topic_id, attrs, REQUEST)

  # Change all attribs of a topic instance, defaults the ones
  # not found in the REQUEST var
  security.declarePublic('editTopicProperties')
  def editTopicProperties(self, forum_id, topic_id, REQUEST):
    fcForumObject.manage_editTopicProperties(self, forum_id, topic_id, REQUEST)

    
  # Membership API: START
  # ===================================================================================    
  # Manage Member's Properties, all these are stored in regular Folder Objects
  # inside the "Members" folder in the ZMI (ZODB)
  #
  # Modules:
  # - setUserProperty()
  # - getUserProperty()
  # - setUserAvatar() --> Included in 2.0.5
  # - getAvatarCode() --> Included in 2.0.5
  #
  security.declarePublic('manage_getUserProperty')
  def manage_getUserProperty(self, property, user, default, REQUEST):
    # lang and locale variables are available
    # to Anonymous Users via a cookie ONLY
    if user[:9] == 'Anonymous':
      user_property = REQUEST.get(property, default)
    else:
      try:
        user_folder = getattr(self.Members, user)
      except AttributeError:
        # Member folder does not exist
        user_folder = ''
        user_property = default

      if user_folder:
        try:
          user_property = getattr(user_folder, property)
        except AttributeError:
          # Member folder exists, but attribute
          # was not found 
          user_property = default

    return user_property  

  security.declarePublic('manage_setUserProperty')
  def manage_setUserProperty(self, property, user, new_value, p_type, REQUEST):

    req = REQUEST
    # Security Update (implemented on v2.0.2)
    # User can only change his/her own properties
    # Unless your role is Manager or Reviewer
    auth_user = req['AUTHENTICATED_USER']

    # One issue with this security implementation is that if the user is signing up
    # there's no way that this method will allow the (still) unauthorized user to
    # modify his TWO properties (user_real_name, user_email), for that I have to do
    # a hack of some sort, where this method will allow to be updated by an "anonymous user"
    # ONLY if she is in the signup process
    new_user       = int(req.get('new_user_flag', 0))
    # This basically will prevent the processing of the new member if
    # se did not supply the (not required) name and email fields
    new_user_flag  = 0
    user_real_name = req.get('user_real_name', '')
    user_email     = req.get('user_email', '')
    if new_user and (user_real_name or user_email) and (property == 'fc_name' or property == 'fc_email'):
      new_user_flag = 1
    # NOTE: IF new_user_flag is set, ONLY (user_real_name, user_email) properties
    # will be available to be added, 
    
    if user == auth_user.getId() or new_user_flag or ('Manager' in auth_user.getRoles() or 'Reviewer' in auth_user.getRoles()):
    
      if p_type:
        prop_type = p_type
      else:
        prop_type = 'string'
    
      # If user Folder inside Member Folder does not exist, create it    
      user_folder_exists = 1
      fatal_error = 0
      prop = property.strip()
      try:
        user_folder = getattr(self.Members, user)
      except AttributeError:
        user_folder_exists = 0
    
      if not user_folder_exists:
        # Create it
        try:
          folder_object = self.Members.manage_addFolder(id=user)
          user_folder   = getattr(self.Members, user)
        except:
          fatal_error = 1

      if not fatal_error:
        property_exists = 1
        # If program gets here, it means that
        # we are sure that the member folder
        # exists, then add/update the specified property
        try:
          user_property = getattr(user_folder, prop)
        except AttributeError:
          # Property does not exist, add it, here we
          # probably want to use some dict with the
          # predefined types, etc, but again, I am a lazy
          # ass, and will "hardcode" the ones that fcForum
          # uses, it I do not understand the property name,
          # then default to string..
          property_exists = 0
          if prop in ['fc_name', 'fc_email', 'fc_subscriptions']:
            user_folder.manage_addProperty(id=prop, type='string', value=new_value)
          elif prop in ['fc_posts', 'fc_allow_email']:
            user_folder.manage_addProperty(id=prop, type='int', value=new_value)
          else:
            # Unknown prop, default to "prop_type"
            user_folder.manage_addProperty(id=prop, type=prop_type, value=new_value)

        if property_exists:
          # This means that user propery already exists, we need to
          # update its value now.. 
          if prop in ['fc_name', 'fc_email', 'fc_subscriptions']:
            # These are overwritten
            req.set(prop, new_value)
          if prop in ['fc_posts', 'fc_allow_email']:
            # This is casted as int
            req.set(prop, int(new_value))
          else: # Default to string
            req.set(prop, new_value)
    
          # manage_editProperties will blank any pros not found in req
          user_folder.manage_changeProperties(req)
    pass

  # Avatar Support - Starting On 2.0.5

  security.declarePublic('getAvatarCode')
  def getAvatarCode(self, avatar_code):
    """Avatar Code Results, URL Callable"""
    code = int(avatar_code)
    avatar_dict = {1: 'Internal Storage Error. Image Cannot be Uploaded, please try again',
                   2: 'Selected Avatar Image Width and Height are both larger than the maximum permissible',
                   3: 'Selected Avatar Image Width is larger than the maximum permissible',
                   4: 'Selected Avatar Image Height is larger than the maximum permissible',
                   5: 'Uploaded File Does not appear to be an Image or the file selected does not exists',
                   6: 'Avatar Container Folder appears to be missing from the system',
                   7: 'Unauthorized to add this avatar at this time'}
    return avatar_dict.get(code, '')

  security.declarePublic('manage_setUserAvatar')
  def manage_setUserAvatar(self, user, avatar, REQUEST):

    # Assumptions:
    # avatar is a VALID file object (that exists)
  
    # Error Codes:
    # 0 - No errors, avatar was successfully created/replaced
    # 1 - Internal Storage Error. Image Cannot be Uploaded, please try again
    # 2 - Selected Image Width and Height are both larger than the maximum permissible
    # 3 - Selected Image Width is larger than the maximum permissible
    # 4 - Selected Image Height is larger than the maximum permissible
    # 5 - Uploaded File Does not appear to be an Image or file does not exists
    # 6 - Avatar Container Folder appears to be missing from the system.
    # 7 - Unauthorized
    

    MAX_HEIGHT  = 200
    MAX_WIDTH   = 100
    MAX_SIZE    = 15000
    err_code    = 0
    fatal_error = 0
    avatar_user = 'avatar_%s' % user
  
    auth_user = REQUEST['AUTHENTICATED_USER']
    
    if user == auth_user.getId() or ('Manager' in auth_user.getRoles() or 'Reviewer' in auth_user.getRoles()):

      # Add the Avatar ONLY if the necessary container folders exist,
      # in a near release, create them if they don't exist..

      if hasattr(self.Members, 'AVATARS'):
        if avatar.headers['Content-Type'].find('image') >= 0:
          try:
            # This will always be deleted after process
            self.Members.AVATARS.temp.manage_addImage(avatar_user, avatar, '')
            # grab the image back
            img_obj = getattr(self.Members.AVATARS.temp, avatar_user)
          except:
            # Most likely the "random" ID already exists (weird)
            # just generate another one on the next try..
            fatal_error = 1
            err_code = 1

          if not fatal_error:
            # Possible height and width errors
            if img_obj.width > MAX_WIDTH and img_obj.height > MAX_HEIGHT:
              err_code = 2
            elif img_obj.width > MAX_WIDTH:
              err_code = 3
            elif img_obj.height > MAX_HEIGHT:
              err_code = 4
            elif len(img_obj) > MAX_SIZE:
              # Possible a hack by the user, thanks to fcarlier for this tip
              err_code = 7
              
            # Continuing Checking
            if err_code:
              # Any Error Just delete the entry
              self.Members.AVATARS.temp.manage_delObjects([avatar_user])
            else:
              # all ok, now "replace" the old (current) avatar with the new one
              # Remove the old One
              if hasattr(self.Members.AVATARS, avatar_user):
                self.Members.AVATARS.manage_delObjects([avatar_user])
              # Rename the new one, aah but of course, renaming in
              # inpossible in this bug world..
              # Move from orig to dest
              img = getattr(self.Members.AVATARS.temp, avatar_user)
              self.Members.AVATARS._setObject(avatar_user, img)
              #self.Members.AVATARS.manage_pasteObjects(self.Members.AVATARS.temp.manage_copyObjects([avatar_user]))
              self.Members.AVATARS.temp.manage_delObjects([avatar_user]) 
          
        else:
          err_code = 5
          
      else:
        # Avatar Folder (Container) Missing
        err_code = 6
    else:
      # Unauthorized
      err_code = 7
          
    return err_code

  security.declarePublic('manage_deleteAvatar')
  def manage_deleteAvatar(self, user, REQUEST):

    # Basic check to prevent someone trying to delete other user's avatar,
    # I am pretty sure there's a more elegant way (i.e. securitywise) to
    # Check for this, but this is for the sake of the dev time.

    avatar_user = 'avatar_%s' % user
  
    auth_user = REQUEST['AUTHENTICATED_USER']
    
    if user == auth_user.getId() or ('Manager' in auth_user.getRoles() or 'Reviewer' in auth_user.getRoles()):
      # Does the avatar/environment actually exists?
      if hasattr(self.Members, 'AVATARS'):
        if hasattr(self.Members.AVATARS, avatar_user):
          self.Members.AVATARS.manage_delObjects([avatar_user])
    pass
    
  # Membership API: END
  # ===================================================================================

  # Forum Addition API
  security.declareProtected('Admin fcForums', 'manage_addForum')
  def manage_addForum(self, forum_title, forum_desc, forum_type, REQUEST):
    """Handles Addition of a forum (folder), addition to "fdesc" property
    and setting permissions"""
    new_id = gen_id()
    self.fcForum[fcForumsLocation].manage_addFolder(new_id, forum_title)
    forum = getattr(self.fcForum[fcForumsLocation], new_id)
    forum.manage_addProperty(id='fdesc', type='text', value=forum_desc)
    if forum_type == 'public':
      forum.manage_permission('Access contents information',[],acquire=1)
      forum.manage_permission('View',[],acquire=1)
    elif forum_type == 'members':
      forum.manage_permission('Access contents information',('Manager', 'Reviewer', 'Member',),acquire=0)
      forum.manage_permission('View',('Manager', 'Reviewer', 'Member',),acquire=0)
    elif forum_type == 'admins':
      forum.manage_permission('Access contents information',('Manager', 'Reviewer',),acquire=0)
      forum.manage_permission('View',('Manager', 'Reviewer',),acquire=0)

  security.declareProtected('Admin fcForums', 'manage_editForum')
  def manage_editForum(self, forum, REQUEST):
    """Handles Editing of a forum object (folder)"""
    forum_obj = getattr(self.fcForum[fcForumsLocation], forum)
    forum_obj.manage_editProperties(REQUEST)

  security.declareProtected('Admin fcForums', 'manage_setForumPermissions')
  def manage_setForumPermissions(self, forum, forum_type):
    """Edits permissions for the forum"""
    forum_obj = getattr(self.fcForum[fcForumsLocation], forum)
    if forum_type == 'public':
      forum_obj.manage_permission('Access contents information',[],acquire=1)
      forum_obj.manage_permission('View',[],acquire=1)
    elif forum_type == 'members':
      forum_obj.manage_permission('Access contents information',('Manager', 'Reviewer', 'Member',),acquire=0)
      forum_obj.manage_permission('View',('Manager', 'Reviewer', 'Member',),acquire=0)
    elif forum_type == 'admins':
      forum_obj.manage_permission('Access contents information',('Manager', 'Reviewer',),acquire=0)
      forum_obj.manage_permission('View',('Manager', 'Reviewer',),acquire=0)
    
  # User Management API
  security.declarePublic('manage_getUserRoles')
  def manage_getUserRoles(self, username, REQUEST):
    if len(username):
      return self.acl_users.getUser(username).getRoles()
    else:
      username = REQUEST['AUTHENTICATED_USER'].getUserName()
      try:
        roles = self.acl_users.getUser(username).getRoles()
      except AttributeError:
        roles = []
      return roles    
  
  security.declarePublic('manage_getUserRole')
  def manage_getUserRole(self, user, default_role, REQUEST):
    if user:
      try:
        roles = self.acl_users.getUser(user).getRoles()
      except AttributeError:
        roles = ['Blah']      
    else:
      roles = REQUEST['AUTHENTICATED_USER'].getRoles()

    if 'Manager' in roles:
      role = 'GodLike'
    elif 'Reviewer' in roles:
      role = 'Forum Admin'
    elif 'Member' in roles:
      role = 'Member'
    else:
      role = default_role
      
    return role 

  security.declarePublic('manage_getUserNames')
  def manage_getUserNames(self):
    try:
      users = self.acl_users.getUserNames()
    except:
      users = []
    return users

  security.declarePublic('manage_getUserRoles')
  def manage_getUserRoles(self, username, REQUEST):
    try:
      all_roles = self.acl_users.getUser(username).getRoles()
    except:
      all_roles = []
    return all_roles

  security.declarePublic('manage_changeUser')
  def manage_changeUser(self, type, REQUEST):
    try:
      self.acl_users.manage_users(submit=type, REQUEST=REQUEST)
    except:
      pass

  security.declarePublic('manage_getRolesOfPermission')
  def manage_getRolesOfPermission(self, forum, permission):
    forum_obj = getattr(self.fcForum[fcForumsLocation], forum)
    return forum_obj.rolesOfPermission(permission)

  security.declareProtected('Admin fcForums', 'manage_changeMailHost')
  def manage_changeMailHost(self, smtp_host, smtp_port, REQUEST):
    """Change SMTP Mail Host/Port"""
    self.MailHost.manage_makeChanges('', smtp_host, int(smtp_port), None)

  security.declarePublic('manage_simpleSendMessage')
  def manage_simpleSendMessage(self, mto, mfrom, subject, body):
    # Return Codes:
    # 0 - Found  Normally Via fcForum MailHost
    # 1 - Found Normally Via 3rd Party Mailhost (Plone, acquired one, etc)
    # 2 - MailHost Not found Local
    # 3 - MailHost Not found External
    # 4 - Error on Mail Server return
    try:
      mailhost = getattr(self, self.superValues('Mail Host')[0].id)
      rval = 0
    except:
      mailhost = None
      rval = 2

    if not mailhost:
      # Hmm, I need to "try" to acquire any mail host that may
      # exist in the acquisition path, I know I can use the __of__
      # to wrap a container from a class, in this case is self, I
      # wonder if I am not alucinating here, the incredible thing
      # is that it appears to work......
      acq_wrapper = self.__of__(self)
      try:
        mailhost = getattr(acq_wrapper, acq_wrapper.superValues('Mail Host')[0].id)
        rval = 1
      except:
        mailhost = None
        rval = 3

    if mailhost:
      try:
        mailhost.simple_send(mto, mfrom, subject, body)
      except: # Connection Refused, Invalid Strings, etc..
        rval = 4
    
    return rval

  security.declareProtected('Admin fcForums', 'manage_delForums')
  def manage_delForums(self, ids):
    """Delete Folder-Style Forums"""
    self.fcForum[fcForumsLocation].manage_delObjects(ids)

  security.declareProtected('Admin fcForums', 'manage_changeForumTitle')
  def manage_changeForumTitle(self, new_title):
    """Changes the main form title"""
    self.manage_changeProperties(title=new_title)

  security.declareProtected('Admin fcForums', 'manage_deleteTopics')
  def manage_deleteTopics(self, forum, topics):
    """
    Safely Removes Topics (fcForumObjects)
    NOTE: For simplicity, this method MUST always
    receive a LIST of topics
    """
    self.fcForum[fcForumsLocation][forum].manage_delObjects(topics)

  security.declarePublic('striphtml')
  def striphtml(self, s, umbr=0):

    class StrippingParser(sgmllib.SGMLParser):

      from htmlentitydefs import entitydefs # replace entitydefs from sgmllib

      def __init__(self, umbr):
          sgmllib.SGMLParser.__init__(self)
          self.result = ""
          self.umbruch = umbr

      def handle_data(self, data):
        if data:
          self.result = self.result + data

      def handle_charref(self, name):
        self.result = "%s&#%s;" % (self.result, name)

      def handle_entityref(self, name):
        if self.entitydefs.has_key(name):
          x = ';'
        else:
          # this breaks unstandard entities that end with ';'
          x = ''
        self.result = "%s&%s%s" % (self.result, name, x)

      def start_br(self, attrs):
        if self.umbruch == 1:
          self.result = self.result + "\n"

      def start_p(self, attrs):
        # muss vorhanden sein, da sonst end_p() ignoriert wird
        pass

      def end_p(self):
        if self.umbruch == 1:
          self.result = self.result + "\n\n"

      def unknown_starttag(self, tag, attrs):
        pass

      def unknown_endtag(self, tag):
        pass


    #def strip(s, umbr=0):
    parser = StrippingParser(umbr)
    parser.feed(s)
    parser.close()
    if umbr == 2:
      return string.join(string.split(parser.result))
    else:
      return parser.result

  security.declarePublic('parseHiddenTags')
  def parseHiddenTags(self, topic_content):
    """
    'removes' the [hidden][/hidden] pseudo-html tag
    attributes
    """
    start_tag = topic_content.find('[hidden]')
    end_tag   = topic_content.rfind('[/hidden]')
    parsed_content = topic_content
    if start_tag >= 0 and end_tag >= 0:
      parsed_content = '%s%s' % (topic_content[:start_tag], topic_content[end_tag + 9:])
    return parsed_content

  security.declarePublic('fixTableBreaks')
  def fixTableBreaks(self, topic_content):
    """
    'removes' the <br /> from [table][/table] pseudo-html tag
    """
    start_tag = topic_content.find('<table')
    end_tag   = topic_content.rfind('</table>')
    parsed_content = topic_content
    if start_tag >= 0 and end_tag >= 0:
      before_table   = parsed_content[:start_tag]
      table_contents = parsed_content[start_tag:end_tag]
      after_table    = parsed_content[end_tag:]
      
      parsed_content = '%s%s%s' % (before_table, table_contents.replace('<br />', '\n'), after_table)
    return parsed_content # (start_tag, end_tag)
    
  # Forum Moderation Support Methods

  # Pending Topics Method
  security.declarePublic('get_pending_topics')
  def get_pending_topics(self, forumId):
    # Returns the number of pending topics to "review"
    # If forum is moderated
    cnt = 0
    topics = self.fcForum[fcForumsLocation][forumId].objectValues('fcForumObject')
    if topics:
      for topic in topics:
        if topic.parent_flag:
          if topic.disabled_flag:
            cnt += 1
    return cnt
 
  security.declarePublic('moderated_check')
  def moderated_check(self, forumId):
    forum = getattr(self.fcForum[fcForumsLocation], forumId)
    if hasattr(forum, 'moderated'):
      # INT
      rval = getattr(forum, 'moderated')
    else:
      rval = 0
    return rval

  security.declareProtected('Admin fcForums', 'moderated_toggle')
  def moderated_toggle(self, forumId, new_value):
    """ Changes Moderation """
    forum = getattr(self.fcForum[fcForumsLocation], forumId)
    if hasattr(forum, 'moderated'):
      m_flag = getattr(forum, 'moderated')
      # Only change if needed..
      if m_flag != new_value:
        forum.manage_changeProperties(moderated=int(new_value))
    else:
      # Attribute does not exist and you want to
      # change, this means: Create it and add the flag
      forum.manage_addProperty(id='moderated', type='int', value=new_value)

    pass

  security.declareProtected('Admin fcForums', 'get_ip_name')
  def get_ip_name(self, ip_addr):
    if ip_addr:
      # socket.gethostbyaddr('216.154.243.185') -> ('www.florescanales.com', [], ['216.154.243.185'])
      try:
        host_name = socket.gethostbyaddr(ip_addr)[0]
      except:
        host_name = '%s [?]' % ip_addr
    else:
      host_name = '%s [x]' % ip_addr
      
    return host_name

  security.declarePublic('re_replace')
  def re_replace(self, msg_str, match_str):
    """ str.replace() on steroids """
    re_expr = ''
    if msg_str:
      for letter in range(len(match_str)):
        letter_l = match_str[letter].lower()
        letter_u = letter_l.upper()
        re_expr += '[%s%s]' % (letter_l, letter_u)
      rval = re.sub(re_expr, '<span style="background-color:yellow;">%s</span>' % match_str, msg_str)
    else:
      rval = ''
    return rval
    
  security.declarePublic('fcwrap')
  def fcwrap(self, text, maxwordlen):
    """Wrap Now"""
    rval = text
    all_text = text.split(' ')
    idx = 0
    for word in all_text:
      if len(word) > maxwordlen:
        wstart = 0
        pos    = maxwordlen
        while 1:
          new_word = word[wstart:pos-1] + ' '
          wstart += maxwordlen
          pos    += maxwordlen
          if wstart >= len(word):
            break
        all_text[idx] = new_word
        idx += 1
    if idx:
      rval = ''
      for word in all_text:
        rval += '%s ' % word
      rval = rval[:-1]
    return rval
    
  security.setPermissionDefault('Admin fcForums', ('Manager', 'Reviewer',))
  
InitializeClass(fcForum)
