__doc__ = """fcForum Topic Object Class"""

__version__ = '1.0.0'

from Globals import DTMLFile
from Globals import InitializeClass
from Globals import Persistent
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import OFS.SimpleItem
import Acquisition
import AccessControl.Role
from DateTime import DateTime
import random
from DocumentTemplate.DT_Var import url_quote, html_quote

# Constructor interface for the fcForumObject. Not actually needed
# since it is kind of "internal" for the forum system, but for
# testing, I'll set one up for now.. in case yyou want to add it as its
# own class instance later
#manage_addFcForumFormObject = PageTemplateFile('www/fcForumObjectAdd.pt', globals())

def manage_addFcForumObject(self, forum_loc, REQUEST):
  """Adds an instance of a forum topic object"""
  id = gen_id()
  self.fcForum.forums[forum_loc]._setObject(id, fcForumObject(id, REQUEST))
  return id

def manage_changeTopicProperties(self, forum_id, topic_id, attrs, REQUEST):
  """Change Properties Specified in a Dictionary"""
  topic_obj = getattr(self.fcForum.forums[forum_id], topic_id)
  for attr in attrs.keys():
    new_value = attrs.get(attr)
    # Persist It
    #try:
    setattr(topic_obj, attr, new_value)
    #except:
    #  topic_obj[attr] = new_value

def manage_editTopicProperties(self, forum_id, topic_id, REQUEST):
  topic_obj = self.fcForum.forums[forum_id][topic_id]
  req = REQUEST.get
  #toppic_obj.id = id (self)
  topic_obj.title = req('title', '')
  topic_obj.content = req('content', '')
  topic_obj.language = req('language', '')
  topic_obj.parent_id = req('parent_id', '')
  topic_obj.parent_flag = req('parent_flag', 0)
  topic_obj.creation_user = req('creation_user', '') # Default Possibly
  topic_obj.creation_date = req('creation_date', DateTime())
  topic_obj.modifying_user = req('modifying_user', '') # Default Possibly
  topic_obj.modifying_date = req('modifying_date', DateTime())
  topic_obj.hits = req('hits', 0)
  topic_obj.locked_flag = req('locked_flag', 0)
  topic_obj.disabled_flag = req('disabled_flag', 0)
  topic_obj.sticky_flag = req('sticky_flag', 0)


class fcForumObject(OFS.SimpleItem.Item,
                    Persistent,
		    Acquisition.Implicit,
		    AccessControl.Role.RoleManager, ):
  """fcForum Object Class Python"""

  # used for the objectValues() function
  meta_type = 'fcForumObject'

  manage_options = (
    {'label': 'Topic Attributes', 'action': 'manage_main'}, )
    
  __ac_permissions__ = (
    ('View management screens', ('manage_tabs', 'manage_main', 'manage_edit')),
    ('Change permissions',      ('manage_access', )),
  )
  
  _properties = (
    {'id':'title', 'type':'string', 'mode':'w'},
    {'id':'content', 'type':'text', 'mode':'w'},
    {'id':'language', 'type':'string', 'mode':'w'},
    {'id':'parent_id', 'type':'string', 'mode':'w'},
    {'id':'parent_flag', 'type':'boolean', 'mode':'w'},
    {'id':'creation_user', 'type':'string', 'mode':'w'},
    {'id':'creation_date', 'type':'date', 'mode':'w'},
    {'id':'modifying_user', 'type':'string', 'mode':'w'},
    {'id':'modifying_date', 'type':'date', 'mode':'w'},
    {'id':'hits', 'type':'int', 'mode':'w'},
    {'id':'locked_flag', 'type':'boolean', 'mode':'w'},
    {'id':'disabled_flag', 'type':'boolean', 'mode':'w'},
    {'id':'sticky_flag  ', 'type':'boolean', 'mode':'w'},
  )

  def __init__(self, id, REQUEST):
    """New Instance Initialization"""
    # Due to time constraints, I'll allow addition
    # of this instance trough-the-web only (via REQUEST)
    req = REQUEST.get
    self.id = id
    self.title = req('title', '')
    self.content = req('content', '')
    self.language = req('language', '')
    self.parent_id = req('parent_id', '')
    self.parent_flag = req('parent_flag', 0)
    self.creation_user = req('creation_user', '') # Default Possibly
    self.creation_date = req('creation_date', DateTime())
    self.modifying_user = req('modifying_user', '') # Default Possibly
    self.modifying_date = req('modifying_date', DateTime())
    self.hits = req('hits', 0)
    self.locked_flag = req('locked_flag', 0)
    self.disabled_flag = req('disabled_flag', 0)
    self.sticky_flag = req('sticky_flag', 0)

  # Object Viewer/Editor
  manage_main = PageTemplateFile('www/editTopicPropertiesForm.pt', globals())
  
  def manage_edit(self, REQUEST=None):
    """Allows Change a forum topic via the ZMI"""
    req = REQUEST.get
    #self.id = id (self)
    self.title = req('new_title', '')
    self.content = req('new_content', '')
    self.language = req('new_language', '')
    self.parent_id = req('new_parent_id', '')
    self.parent_flag = req('new_parent_flag', 0)
    self.creation_user = req('new_creation_user', '') # Default Possibly
    self.creation_date = req('new_creation_date', DateTime())
    self.modifying_user = req('new_modifying_user', '') # Default Possibly
    self.modifying_date = req('new_modifying_date', DateTime())
    self.hits = req('new_hits', 0)
    self.locked_flag = req('new_locked_flag', 0)
    self.disabled_flag = req('new_disabled_flag', 0)
    self.sticky_flag = req('new_sticky_flag', 0)
    if REQUEST is not None:
      return self.manage_main()
    
def gen_id():
  return str(random.random())[2:]


InitializeClass(fcForumObject)
